//+------------------------------------------------------------------+
//|                                                        Clock.mq4 |
//|                                                           Jerome |
//|                                                4xCoder@gmail.com |
//+------------------------------------------------------------------+
//------------------------------------------------------------------
// Instructions
//    This Version requires Allow DLL Imports be set in Common Tab when you add this to a chart.
//    You can also enable this by default in the Options>Expert Advisors Tab, but you may want
//    to turn off "Confirm DLL Function Calls"
//
//    ShowLocal - Set to tru to show your local time zone
//    corner    - 0 = top left, 1 = top right, 2 = bottom left, 3 = bottom right
//    topOff    - pixels from top to show the clock
//    labelColor- Color of label
//    clockColor- Color of clock
//    show12HourTime - true show 12 hour time, false, show 24 hour time
//

#property copyright "Jerome"
#property link      "4xCoder@gmail.com"

#property indicator_chart_window

//
//
//
//
//

#import "kernel32.dll"
void GetLocalTime(int& TimeArray[]);
void GetSystemTime(int& TimeArray[]);
int  GetTimeZoneInformation(int& TZInfoArray[]);
#import

//
//
//
//
//

extern bool   ShowLocal      = true;
extern int    corner         = 1;
extern int    topOff         = 120;
extern color  labelColor     = DarkKhaki;
extern color  clockColor     = DarkKhaki;
extern bool   show12HourTime = false;

//
//
//
//
//

int LondonTZ  = 0;
int TokyoTZ   = 9;
int NewYorkTZ = -5;

//
//
//
//
//

int init()
{

   
   int top  = topOff;
   int left = 90;
   
   //
   //
   //
   //
   //
   
   if (show12HourTime) left = 102;
   if (ShowLocal) 
   {
      ObjectMakeLabel("locl",            left,top);
      ObjectMakeLabel("loct",            45,  top);
   }
   
      ObjectMakeLabel("gmtl",            left,top-15);
      ObjectMakeLabel("gmtt",            45,  top-15);
      ObjectMakeLabel("nyl",             left,top-30);
      ObjectMakeLabel("nyt",             45,  top-30);
      ObjectMakeLabel("lonl",            left,top-45);
      ObjectMakeLabel("lont",            45,  top-45);
      ObjectMakeLabel("tokl",            left,top-60);
      ObjectMakeLabel("tokt",            45,  top-60);
      ObjectMakeLabel("brol",            left,top-75);
      ObjectMakeLabel("brot",            45,  top-75);
      ObjectMakeLabel("barl",            left,top-90);
      ObjectMakeLabel("bart",            45,  top-90);
      ObjectMakeLabel("Spread Monitor1", left,top-105);   
      ObjectMakeLabel("Spread Monitor2", 45,  top-105); 
        
   return(0);
  }
  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int deinit()
{

   ObjectDelete( "locl" );
   ObjectDelete( "loct" );
   ObjectDelete( "nyl" );
   ObjectDelete( "nyt" );
   ObjectDelete( "gmtl" );
   ObjectDelete( "gmtt" );
   ObjectDelete( "lonl" );
   ObjectDelete( "lont" );
   ObjectDelete( "tokl" );
   ObjectDelete( "tokt" );
   ObjectDelete( "brol" );
   ObjectDelete( "brot" );
   ObjectDelete( "barl" );
   ObjectDelete( "bart" );
   ObjectDelete( "Spread Monitor1" );   
   ObjectDelete( "Spread Monitor2" );       

   return(0);
  }
  
//
//
//
//
//

int start()
{
  if (!IsDllsAllowed()) 
  {
      Alert("Clock V1_2: DLLs are disabled.  To enable tick the checkbox in the Common Tab of indicator");
      return;
  }
      
   int    TimeArray[4];
   int    TZInfoArray[43];
   string sMilliSec;
   
   GetLocalTime(TimeArray);

   int      nYear      = TimeArray[0]&0x0000FFFF;
   int      nMonth     = TimeArray[0]>>16;
   int      nDay       = TimeArray[1]>>16;
   int      nHour      = TimeArray[2]&0x0000FFFF;
   int      nMin       = TimeArray[2]>>16;
   int      nSec       = TimeArray[3]&0x0000FFFF;
   int      nMilliSec  = TimeArray[3]>>16;
   string   LocalTimeS = FormatDateTime(nYear,nMonth,nDay,nHour,nMin,nSec);
   datetime localTime  = StrToTime( LocalTimeS );

   //-----------------------------------------------------
   LondonTZ  = GMT_Offset("LONDON",localTime);
   TokyoTZ   = GMT_Offset("TOKYO", localTime);
   NewYorkTZ = GMT_Offset("US",    localTime);
   //-----------------------------------------------------

   int gmt_shift = -480;
   int dst       = GetTimeZoneInformation(TZInfoArray);
   
   if(dst!=0) gmt_shift  = TZInfoArray[0];
   if(dst==2) gmt_shift += TZInfoArray[42];
   

   datetime brokerTime = CurTime();
   datetime GMT        = localTime + gmt_shift * 60;
   
   datetime london  = GMT + (LondonTZ)  * 3600;
   datetime tokyo   = GMT + (TokyoTZ)   * 3600;
   datetime newyork = GMT + (NewYorkTZ) * 3600;
   
   string GMTs     = TimeToString(GMT);
   string locals   = TimeToString(localTime);
   string londons  = TimeToString(london);
   string tokyos   = TimeToString(tokyo);
   string newyorks = TimeToString(newyork);
   string brokers  = TimeToString(CurTime());
   string bars     = TimeToStr(CurTime() - Time[0], TIME_MINUTES);
   string symbol   = Symbol(); if (StringSubstr(symbol,0,2)=="_t") symbol = StringSubstr(symbol,2);
   int    digits   = MarketInfo(symbol,MODE_DIGITS);
   double modifier = 1;
   if (digits==3 || digits==5) modifier = 10.0;
   
   if (ShowLocal) 
   {
      ObjectSetText("locl",            "Local:",   10, "Arial", labelColor);
      ObjectSetText("loct",            locals,     10, "Arial", clockColor);
   }
      ObjectSetText("gmtl",            "GMT:",     10, "Arial", labelColor);
      ObjectSetText("gmtt",            GMTs,       10, "Arial", clockColor);
      ObjectSetText("nyl",             "New York:",10, "Arial", labelColor);
      ObjectSetText("nyt",             newyorks,   10, "Arial", clockColor);
      ObjectSetText("lonl",            "London:",  10, "Arial", labelColor);
      ObjectSetText("lont",            londons,    10, "Arial", clockColor);
      ObjectSetText("tokl",            "Tokyo:",   10, "Arial", labelColor);
      ObjectSetText("tokt",            tokyos,     10, "Arial", clockColor);
      ObjectSetText("brol",            "Broker:",  10, "Arial", labelColor);
      ObjectSetText("brot",            brokers,    10, "Arial", clockColor);
      ObjectSetText("barl",            "Bar:",     10, "Arial", labelColor);
      ObjectSetText("bart",            bars,       10, "Arial", clockColor);
      ObjectSetText("Spread Monitor1", "Spread:",  10, "Arial", labelColor);
      ObjectSetText("Spread Monitor2",DoubleToStr(MarketInfo(symbol,MODE_SPREAD)/modifier,2),   10, "Arial", clockColor);

return(0);
}

//
//
//
//
//

string TimeToString(datetime when) 
{
   if (!show12HourTime) return (TimeToStr(when, TIME_MINUTES));
      
   int hour   = TimeHour(when);
   int minute = TimeMinute(when);
   
   string ampm = " AM";
   string timeStr;
   if (hour >= 12) { hour = hour - 12; ampm = " PM"; }
      
   if (hour == 0) hour = 12; timeStr = DoubleToStr(hour,0) + ":";
   if (minute < 10)          
   timeStr = timeStr + "0"; timeStr = timeStr + DoubleToStr(minute,0); timeStr = timeStr + ampm;
   
return (timeStr);
}

//
//
//
//
//

int ObjectMakeLabel( string n, int xoff, int yoff) 
{
   ObjectCreate(n, OBJ_LABEL, 0, 0, 0);
      ObjectSet(n, OBJPROP_CORNER, corner);
      ObjectSet(n, OBJPROP_XDISTANCE, xoff);
      ObjectSet(n, OBJPROP_YDISTANCE, yoff);
      ObjectSet(n, OBJPROP_BACK, true);
}

//
//
//
//
//

string FormatDateTime(int nYear,int nMonth,int nDay,int nHour,int nMin,int nSec)
{
   string sMonth,sDay,sHour,sMin,sSec;
   
   //
   //
   //
   //
   //
   
   sMonth = 100+nMonth;
   sMonth = StringSubstr(sMonth,1);
   sDay   = 100+nDay;
   sDay   = StringSubstr(sDay,1);
   sHour  = 100+nHour;
   sHour  = StringSubstr(sHour,1);
   sMin   = 100+nMin;
   sMin   = StringSubstr(sMin,1);
   sSec   = 100+nSec;
   sSec   = StringSubstr(sSec,1);

return(StringConcatenate(nYear,".",sMonth,".",sDay," ",sHour,":",sMin,":",sSec));
}

//
//
//
//
//

int GMT_Offset(string region,datetime dt1)
{
  int r1=0;
  
  //
  //
  //
  //
  //
  
       if (region == "FRANKFURT")  r1 = GMT1(dt1);  
  else if (region == "LONDON")     r1 = GMT0(dt1);
  else if (region == "US")         r1 = GMT_5(dt1); 
  else if (region == "TOKYO")      r1 = GMT9(dt1);      
return (r1);

}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GMT0(datetime dt1)
{

  if ((dt1>last_sunday(TimeYear(dt1),3))&&(dt1<last_sunday(TimeYear(dt1),10)))
    return(1);//summer
  else
return(0); 
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GMT_5(datetime dt1)
{
                    

 if(TimeYear(dt1)<2007)
   if ((dt1>first_sunday(TimeYear(dt1),4))&&(dt1<last_sunday(TimeYear(dt1),10)))
     return(-4);
    else
     return(-5); 
 else
   if ((dt1>second_sunday(TimeYear(dt1),3))&&(dt1<first_sunday(TimeYear(dt1),11)))
     return(-4);
    else
     return(-5); 
     
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GMT9(datetime dt1)
{
   return(9);//standard=summer=+9

}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

bool is_leap_year(int year1)
{
 
  if ((MathMod(year1,100)==0) && (MathMod(year1,400)==0))
    return(true);
    
  else if ((MathMod(year1,100)!=0) && (MathMod(year1,4)==0))  
    return(true);
    
  else 
return (false); 
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int n_days(int year1,int month1)
{
  int ndays1;
  
  //
  //
  //
  //
  //
  
  if (month1==1 || month1==3 || month1==5 || month1==7 || month1==8 || month1==10 || month1==12)
    ndays1=31;
    
  else if(month1==4 || month1==6 || month1==9 || month1==11)
    ndays1=30; 
    
  else if(month1==2)
  {
    if (is_leap_year(year1))
      ndays1=29;      
    else
      ndays1=28;  
  }    
   
return(ndays1);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int n_sdays(int year1,int month1)
{
  datetime ddt2;
  int ndays2=n_days(year1,month1);
  int i,nsun1=0;  
  for (i=1;i<=ndays2;i++) 
  {
    ddt2= StrToTime(DoubleToStr(year1,0)+"."+DoubleToStr(month1,0)+"."+DoubleToStr(i,0)+" 00:00");            
    if(TimeDayOfWeek(ddt2)==0)
      nsun1=nsun1+1; 
  }   
  return(nsun1);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

datetime last_sunday(int year1,int month1)
{
  int i,ndays2,nsun1,nsun2;
  datetime dt2,dt3;
  
  //
  //
  //
  //
  //
  
  ndays2=n_days(year1,month1);
  nsun2=n_sdays(year1,month1);
  nsun1=0;
  for (i=1;i<=ndays2;i++) 
  {
    dt2= StrToTime(DoubleToStr(year1,0)+"."+DoubleToStr(month1,0)+"."+DoubleToStr(i,0)+" 00:00");                    
    if(TimeDayOfWeek(dt2)==0)
    {       
      nsun1=nsun1+1; 
    }  
    if (nsun1==nsun2) 
    {
      dt3=dt2;  
      break;        
    }  
  }   
  return(dt3);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

datetime first_sunday(int year1,int month1)
{
  int i,ndays2,nsun1,nsun2;
  datetime dt2,dt3;
  
  //
  //
  //
  //
  //
  
  ndays2=n_days(year1,month1);
  nsun2=1;//n_sdays(year1,month1);
  nsun1=0;
  for (i=1;i<=ndays2;i++) 
  {
    dt2= StrToTime(DoubleToStr(year1,0)+"."+DoubleToStr(month1,0)+"."+DoubleToStr(i,0)+" 00:00");                    
    if(TimeDayOfWeek(dt2)==0)
    {       
      nsun1=nsun1+1; 
    }  
    if (nsun1==nsun2) 
    {
      dt3=dt2;  
      break;        
    }  
  }   
  return(dt3);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

datetime second_sunday(int year1,int month1)
{
  int i,ndays2,nsun1,nsun2;
  datetime dt2,dt3;
  
  //
  ///
  //
  //
  
  ndays2=n_days(year1,month1);
  nsun2=2;//n_sdays(year1,month1);
  nsun1=0;
  for (i=1;i<=ndays2;i++) 
  {
    dt2= StrToTime(DoubleToStr(year1,0)+"."+DoubleToStr(month1,0)+"."+DoubleToStr(i,0)+" 00:00");                    
    if(TimeDayOfWeek(dt2)==0)
    {       
      nsun1=nsun1+1; 
    }  
    if (nsun1==nsun2) 
    {
      dt3=dt2;  
      break;        
    }  
  }   
  return(dt3);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
///

int GMT1(datetime dt1)
{
  
  if ((dt1>last_sunday(TimeYear(dt1),3))&&(dt1<last_sunday(TimeYear(dt1),10)))
   return(2);
  else
   return(1); 
}

  

